@@ -22,11 +22,31 @@
 
 import types
 
+import six
+
+
+if six.PY3:
+    _NAME_TYPES = (
+        types.ModuleType,
+        types.FunctionType,
+        types.LambdaType,
+        type,
+    )
+else:
+    _NAME_TYPES = (
+        types.TypeType,
+        types.ModuleType,
+        types.FunctionType,
+        types.LambdaType,
+        types.ClassType,
+    )
+
 
 def obj_name(obj):
-    if isinstance(obj, (types.TypeType,
-                        types.ModuleType,
-                        types.FunctionType,
-                        types.LambdaType)):
-        return str(obj.__name__)
-    return obj_name(obj.__class__)
+    if isinstance(obj, _NAME_TYPES):
+        return six.text_type(obj.__name__)
+    else:
+        if not hasattr(obj, '__class__'):
+            return repr(obj)
+        else:
+            return obj_name(obj.__class__)
